@@ -61,15 +61,14 @@ send_message (GObject *object,
       goto finally;
     }
 
-  received = tp_message_new (tp_base_channel_get_connection (base), 1, len);
+  received = tp_cm_message_new (tp_base_channel_get_connection (base), 1);
 
   /* Copy/modify the headers for the "received" message */
     {
       TpChannelTextMessageType message_type;
       gboolean valid;
 
-      tp_message_set_handle (received, 0, "message-sender",
-          TP_HANDLE_TYPE_CONTACT,
+      tp_cm_message_set_sender (received,
           tp_base_channel_get_target_handle (base));
 
       message_type = tp_asv_get_uint32 (tp_message_peek (message, 0),
@@ -172,7 +171,7 @@ constructor (GType type,
       tp_base_channel_get_connection (base));
 
   tp_message_mixin_implement_sending (object, send_message,
-      (sizeof (types) / sizeof (types[0])), types,
+      G_N_ELEMENTS (types), types,
       TP_MESSAGE_PART_SUPPORT_FLAG_ONE_ATTACHMENT |
       TP_MESSAGE_PART_SUPPORT_FLAG_MULTIPLE_ATTACHMENTS,
       TP_DELIVERY_REPORTING_SUPPORT_FLAG_RECEIVE_FAILURES,
@@ -215,6 +214,24 @@ example_echo_2_channel_close (TpBaseChannel *self)
     }
 }
 
+static void
+example_echo_2_channel_fill_immutable_properties (TpBaseChannel *chan,
+    GHashTable *properties)
+{
+  TpBaseChannelClass *klass = TP_BASE_CHANNEL_CLASS (
+      example_echo_2_channel_parent_class);
+
+  klass->fill_immutable_properties (chan, properties);
+
+  tp_dbus_properties_mixin_fill_properties_hash (
+      G_OBJECT (chan), properties,
+      TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "MessagePartSupportFlags",
+      TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "DeliveryReportingSupport",
+      TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "SupportedContentTypes",
+      TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "MessageTypes",
+      NULL);
+}
+
 static void
 example_echo_2_channel_class_init (ExampleEcho2ChannelClass *klass)
 {
@@ -228,6 +245,8 @@ example_echo_2_channel_class_init (ExampleEcho2ChannelClass *klass)
   base_class->target_handle_type = TP_HANDLE_TYPE_CONTACT;
   base_class->interfaces = example_echo_2_channel_interfaces;
   base_class->close = example_echo_2_channel_close;
+  base_class->fill_immutable_properties =
+    example_echo_2_channel_fill_immutable_properties;
 
   tp_message_mixin_init_dbus_properties (object_class);
 }
