@@ -19,6 +19,16 @@ G_DEFINE_TYPE (ExampleEcho2Protocol,
     example_echo_2_protocol,
     TP_TYPE_BASE_PROTOCOL)
 
+const gchar * const protocol_interfaces[] = {
+  TP_IFACE_PROTOCOL_INTERFACE_AVATARS,
+  NULL };
+
+const gchar * const supported_avatar_mime_types[] = {
+  "image/png",
+  "image/jpeg",
+  "image/gif",
+  NULL };
+
 static void
 example_echo_2_protocol_init (
     ExampleEcho2Protocol *self)
@@ -107,7 +117,7 @@ identify_account (TpBaseProtocol *self G_GNUC_UNUSED,
 static GStrv
 get_interfaces (TpBaseProtocol *self)
 {
-  return NULL;
+  return g_strdupv ((GStrv) protocol_interfaces);
 }
 
 static void
@@ -152,6 +162,42 @@ get_connection_details (TpBaseProtocol *self G_GNUC_UNUSED,
     }
 }
 
+static void
+get_avatar_details (TpBaseProtocol *self,
+    GStrv *supported_mime_types,
+    guint *min_height,
+    guint *min_width,
+    guint *recommended_height,
+    guint *recommended_width,
+    guint *max_height,
+    guint *max_width,
+    guint *max_bytes)
+{
+  if (supported_mime_types != NULL)
+    *supported_mime_types = g_strdupv ((GStrv) supported_avatar_mime_types);
+
+  if (min_height != NULL)
+    *min_height = 32;
+
+  if (min_width != NULL)
+    *min_width = 32;
+
+  if (recommended_height != NULL)
+    *recommended_height = 64;
+
+  if (recommended_width != NULL)
+    *recommended_width = 64;
+
+  if (max_height != NULL)
+    *max_height = 96;
+
+  if (max_width != NULL)
+    *max_width = 96;
+
+  if (max_bytes != NULL)
+    *max_bytes = 37748736;
+}
+
 static void
 example_echo_2_protocol_class_init (
     ExampleEcho2ProtocolClass *klass)
@@ -166,4 +212,5 @@ example_echo_2_protocol_class_init (
   base_class->identify_account = identify_account;
   base_class->get_interfaces = get_interfaces;
   base_class->get_connection_details = get_connection_details;
+  base_class->get_avatar_details = get_avatar_details;
 }
