@@ -528,7 +528,7 @@ out:
 }
 
 static void
-channel_prepare_cb (GObject *source,
+adoc_channel_prepare_cb (GObject *source,
     GAsyncResult *result,
     gpointer user_data)
 {
@@ -553,12 +553,12 @@ out:
 }
 
 static void
-context_prepare (TpAddDispatchOperationContext *self)
+context_prepare (TpAddDispatchOperationContext *self,
+    const GQuark *account_features,
+    const GQuark *connection_features,
+    const GQuark *channel_features)
 {
-  GQuark account_features[] = { TP_ACCOUNT_FEATURE_CORE, 0 };
-  GQuark conn_features[] = { TP_CONNECTION_FEATURE_CORE, 0 };
   GQuark cdo_features[] = { TP_CHANNEL_DISPATCH_OPERATION_FEATURE_CORE, 0 };
-  GQuark channel_features[] = { TP_CHANNEL_FEATURE_CORE, 0 };
   guint i;
 
   self->priv->num_pending = 3;
@@ -566,7 +566,7 @@ context_prepare (TpAddDispatchOperationContext *self)
   tp_proxy_prepare_async (self->account, account_features,
       account_prepare_cb, g_object_ref (self));
 
-  tp_proxy_prepare_async (self->connection, conn_features,
+  tp_proxy_prepare_async (self->connection, connection_features,
       conn_prepare_cb, g_object_ref (self));
 
   tp_proxy_prepare_async (self->dispatch_operation, cdo_features,
@@ -579,13 +579,16 @@ context_prepare (TpAddDispatchOperationContext *self)
       self->priv->num_pending++;
 
       tp_proxy_prepare_async (channel, channel_features,
-          channel_prepare_cb, g_object_ref (self));
+          adoc_channel_prepare_cb, g_object_ref (self));
     }
 }
 
 void
 _tp_add_dispatch_operation_context_prepare_async (
     TpAddDispatchOperationContext *self,
+    const GQuark *account_features,
+    const GQuark *connection_features,
+    const GQuark *channel_features,
     GAsyncReadyCallback callback,
     gpointer user_data)
 {
@@ -597,7 +600,8 @@ _tp_add_dispatch_operation_context_prepare_async (
   self->priv->result = g_simple_async_result_new (G_OBJECT (self),
       callback, user_data, _tp_add_dispatch_operation_context_prepare_async);
 
-  context_prepare (self);
+  context_prepare (self, account_features, connection_features,
+      channel_features);
 }
 
 gboolean
