@@ -777,7 +777,7 @@ tp_base_client_add_handler_capabilities_varargs (TpBaseClient *self,
     const gchar *first_token, ...)
 {
   va_list ap;
-  const gchar *token = first_token;
+  const gchar *token;
 
   g_return_if_fail (TP_IS_BASE_CLIENT (self));
   g_return_if_fail (!self->priv->registered);
