@@ -29,6 +29,7 @@
 #include <telepathy-glib/defs.h>
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/handle-repo.h>
+#include <telepathy-glib/proxy.h>
 #include <telepathy-glib/svc-connection.h>
 
 G_BEGIN_DECLS
@@ -58,7 +59,13 @@ typedef gchar *(*TpBaseConnectionGetUniqueConnectionNameImpl) (
 struct _TpBaseConnectionClass {
     GObjectClass parent_class;
 
+#ifdef __GI_SCANNER__
+    /*<private>*/
+    GCallback _internal_create_handle_repos;
+    /*<public>*/
+#else
     TpBaseConnectionCreateHandleReposImpl create_handle_repos;
+#endif
 
     TpBaseConnectionCreateChannelFactoriesImpl create_channel_factories;
 
@@ -192,6 +199,15 @@ gboolean tp_base_connection_channel_manager_iter_next (
       } \
   } G_STMT_END
 
+TpDBusDaemon *tp_base_connection_get_dbus_daemon (TpBaseConnection *self);
+
+void tp_base_connection_add_client_interest (TpBaseConnection *self,
+    const gchar *unique_name, const gchar *token,
+    gboolean only_if_uninterested);
+
+void tp_base_connection_add_possible_client_interest (TpBaseConnection *self,
+    GQuark token);
+
 G_END_DECLS
 
 #endif /* #ifndef __TP_BASE_CONNECTION_H__*/
