@@ -23,6 +23,7 @@
 #define __TP_CHANNEL_H__
 
 #include <telepathy-glib/connection.h>
+#include <telepathy-glib/defs.h>
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/handle.h>
 #include <telepathy-glib/intset.h>
@@ -44,6 +45,7 @@ struct _TpChannelClass {
 };
 
 struct _TpChannel {
+    /*<private>*/
     TpProxy parent;
 
     TpChannelPrivate *priv;
@@ -80,8 +82,10 @@ TpChannel *tp_channel_new_from_properties (TpConnection *conn,
     const gchar *object_path, const GHashTable *immutable_properties,
     GError **error) G_GNUC_WARN_UNUSED_RESULT;
 
+#ifndef TP_DISABLE_DEPRECATED
 gboolean tp_channel_run_until_ready (TpChannel *self, GError **error,
-    GMainLoop **loop);
+    GMainLoop **loop) _TP_GNUC_DEPRECATED;
+#endif
 
 typedef void (*TpChannelWhenReadyCb) (TpChannel *channel, const GError *error,
     gpointer user_data);
@@ -101,15 +105,53 @@ GHashTable *tp_channel_borrow_immutable_properties (TpChannel *self);
 
 TpHandle tp_channel_group_get_self_handle (TpChannel *self);
 TpChannelGroupFlags tp_channel_group_get_flags (TpChannel *self);
-const TpIntSet *tp_channel_group_get_members (TpChannel *self);
-const TpIntSet *tp_channel_group_get_local_pending (TpChannel *self);
-const TpIntSet *tp_channel_group_get_remote_pending (TpChannel *self);
+const TpIntset *tp_channel_group_get_members (TpChannel *self);
+const TpIntset *tp_channel_group_get_local_pending (TpChannel *self);
+const TpIntset *tp_channel_group_get_remote_pending (TpChannel *self);
 gboolean tp_channel_group_get_local_pending_info (TpChannel *self,
     TpHandle local_pending, TpHandle *actor,
     TpChannelGroupChangeReason *reason, const gchar **message);
 
 TpHandle tp_channel_group_get_handle_owner (TpChannel *self, TpHandle handle);
 
+gboolean tp_channel_get_requested (TpChannel *self);
+
+TpHandle tp_channel_get_initiator_handle (TpChannel *self);
+const gchar * tp_channel_get_initiator_identifier (TpChannel *self);
+
+#define TP_CHANNEL_FEATURE_CORE \
+  tp_channel_get_feature_quark_core ()
+
+GQuark tp_channel_get_feature_quark_core (void) G_GNUC_CONST;
+
+#define TP_CHANNEL_FEATURE_GROUP \
+  tp_channel_get_feature_quark_group ()
+GQuark tp_channel_get_feature_quark_group (void) G_GNUC_CONST;
+
+#define TP_CHANNEL_FEATURE_CHAT_STATES \
+  tp_channel_get_feature_quark_chat_states ()
+GQuark tp_channel_get_feature_quark_chat_states (void) G_GNUC_CONST;
+TpChannelChatState tp_channel_get_chat_state (TpChannel *self,
+    TpHandle contact);
+
+void tp_channel_leave_async (TpChannel *self,
+    TpChannelGroupChangeReason reason,
+    const gchar *message,
+    GAsyncReadyCallback callback,
+    gpointer user_data);
+
+gboolean tp_channel_leave_finish (TpChannel *self,
+    GAsyncResult *result,
+    GError **error);
+
+void tp_channel_close_async (TpChannel *self,
+    GAsyncReadyCallback callback,
+    gpointer user_data);
+
+gboolean tp_channel_close_finish (TpChannel *self,
+    GAsyncResult *result,
+    GError **error);
+
 G_END_DECLS
 
 #include <telepathy-glib/_gen/tp-cli-channel.h>
