@@ -23,22 +23,22 @@
 #define __TELEPATHY_DBUS_H__
 #define __TP_IN_DBUS_H__
 
+#include <telepathy-glib/defs.h>
 #include <telepathy-glib/dbus-daemon.h>
 
+#include <telepathy-glib/_gen/genums.h>
+
 G_BEGIN_DECLS
 
 void tp_dbus_g_method_return_not_implemented (DBusGMethodInvocation *context);
-DBusGConnection * tp_get_bus (void);
 
-typedef enum
+typedef enum /*< flags >*/
 {
   TP_DBUS_NAME_TYPE_UNIQUE = 1,
   TP_DBUS_NAME_TYPE_WELL_KNOWN = 2,
   TP_DBUS_NAME_TYPE_BUS_DAEMON = 4,
-  TP_DBUS_NAME_TYPE_NOT_BUS_DAEMON =
-    TP_DBUS_NAME_TYPE_UNIQUE | TP_DBUS_NAME_TYPE_WELL_KNOWN,
-  TP_DBUS_NAME_TYPE_ANY =
-    TP_DBUS_NAME_TYPE_NOT_BUS_DAEMON | TP_DBUS_NAME_TYPE_BUS_DAEMON
+  TP_DBUS_NAME_TYPE_NOT_BUS_DAEMON = TP_DBUS_NAME_TYPE_UNIQUE | TP_DBUS_NAME_TYPE_WELL_KNOWN,
+  TP_DBUS_NAME_TYPE_ANY = TP_DBUS_NAME_TYPE_NOT_BUS_DAEMON | TP_DBUS_NAME_TYPE_BUS_DAEMON
 } TpDBusNameType;
 
 gboolean tp_dbus_check_valid_bus_name (const gchar *name,
@@ -127,7 +127,8 @@ void tp_asv_set_strv (GHashTable *asv, const gchar *key, gchar **value);
 void tp_asv_dump (GHashTable *asv);
 
 #ifndef TP_DISABLE_DEPRECATED
-DBusGProxy * tp_get_bus_proxy (void) G_GNUC_DEPRECATED;
+DBusGConnection * tp_get_bus (void) _TP_GNUC_DEPRECATED;
+DBusGProxy * tp_get_bus_proxy (void) _TP_GNUC_DEPRECATED;
 #endif
 
 G_END_DECLS
