@@ -122,13 +122,12 @@ local_pending_info_new (TpHandleRepoIface *repo,
                         const gchar *message)
 {
   LocalPendingInfo *info = g_slice_new0 (LocalPendingInfo);
-  info->actor = actor;
   info->reason = reason;
   info->message = g_strdup (message);
   info->repo = repo;
 
   if (actor != 0)
-    tp_handle_ref (repo, actor);
+    info->actor = tp_handle_ref (repo, actor);
 
   return info;
 }
@@ -156,6 +155,27 @@ struct _TpGroupMixinPrivate {
     GPtrArray *externals;
 };
 
+/**
+ * TP_HAS_GROUP_MIXIN:
+ * @o: a #GObject instance
+ *
+ * <!-- -->
+ *
+ * Returns: %TRUE if @o (or one of its parent classes) has the group mixin.
+ *
+ * Since: 0.13.9
+ */
+
+/**
+ * TP_HAS_GROUP_MIXIN_CLASS:
+ * @cls: a #GObjectClass structure
+ *
+ * <!-- -->
+ *
+ * Returns: %TRUE if @cls (or one of its parent classes) has the group mixin.
+ *
+ * Since: 0.13.9
+ */
 
 /**
  * tp_group_mixin_class_get_offset_quark: (skip)
@@ -310,10 +330,9 @@ tp_group_mixin_init (GObject *obj,
   mixin = TP_GROUP_MIXIN (obj);
 
   mixin->handle_repo = handle_repo;
-  mixin->self_handle = self_handle;
 
-  if (mixin->self_handle != 0)
-    tp_handle_ref (handle_repo, mixin->self_handle);
+  if (self_handle != 0)
+    mixin->self_handle = tp_handle_ref (handle_repo, self_handle);
 
   mixin->group_flags = 0;
 
@@ -445,10 +464,11 @@ tp_group_mixin_change_self_handle (GObject *obj,
   DEBUG ("%u '%s'", new_self_handle,
       tp_handle_inspect (mixin->handle_repo, new_self_handle));
 
-  if (new_self_handle != 0)
-    tp_handle_ref (mixin->handle_repo, new_self_handle);
+  mixin->self_handle = 0;
 
-  mixin->self_handle = new_self_handle;
+  if (new_self_handle != 0)
+    mixin->self_handle = tp_handle_ref (mixin->handle_repo,
+        new_self_handle);
 
   tp_svc_channel_interface_group_emit_self_handle_changed (obj,
       new_self_handle);
@@ -720,8 +740,7 @@ tp_group_mixin_remove_members_with_reason (GObject *obj,
         {
           DEBUG ("handle %u is not a current or pending member",
                    handle);
-          /* skip this handle during the second pass */
-          g_array_index (contacts, TpHandle, i) = 0;
+          /* we'll skip this handle during the second pass */
         }
     }
 
@@ -730,7 +749,9 @@ tp_group_mixin_remove_members_with_reason (GObject *obj,
     {
       handle = g_array_index (contacts, TpHandle, i);
 
-      if (handle == 0)
+      if (!tp_handle_set_is_member (mixin->members, handle) &&
+          !tp_handle_set_is_member (mixin->remote_pending, handle) &&
+          !tp_handle_set_is_member (mixin->local_pending, handle))
         continue;
 
       if (mixin_cls->priv->remove_with_reason != NULL)
@@ -1177,7 +1198,7 @@ group_flags_to_string (TpChannelGroupFlags flags)
   gint i = 0;
   GString *str;
 
-  str = g_string_new ("[" TP_ANSI_BOLD_OFF);
+  str = g_string_new ("[");
 
   GFTS_APPEND_FLAG_IF_SET (TP_CHANNEL_GROUP_FLAG_CAN_ADD);
   GFTS_APPEND_FLAG_IF_SET (TP_CHANNEL_GROUP_FLAG_CAN_REMOVE);
@@ -1204,7 +1225,7 @@ group_flags_to_string (TpChannelGroupFlags flags)
       g_string_append_printf (str, "%u", flags);
     }
 
-  g_string_append (str, TP_ANSI_BOLD_ON "]");
+  g_string_append (str, "]");
 
   return g_string_free (str, FALSE);
 }
@@ -1255,11 +1276,10 @@ tp_group_mixin_change_flags (GObject *obj,
           str_removed = group_flags_to_string (removed);
           str_flags = group_flags_to_string (mixin->group_flags);
 
-          printf (TP_ANSI_BOLD_ON TP_ANSI_FG_WHITE
-                  "%s: emitting group flags changed\n"
+          printf ("%s: emitting group flags changed\n"
                   "  added    : %s\n"
                   "  removed  : %s\n"
-                  "  flags now: %s\n" TP_ANSI_RESET,
+                  "  flags now: %s\n",
                   G_STRFUNC, str_added, str_removed, str_flags);
 
           fflush (stdout);
@@ -1292,7 +1312,7 @@ member_array_to_string (TpHandleRepoIface *repo,
   GString *str;
   guint i;
 
-  str = g_string_new ("[" TP_ANSI_BOLD_OFF);
+  str = g_string_new ("[");
 
   for (i = 0; i < array->len; i++)
     {
@@ -1309,7 +1329,7 @@ member_array_to_string (TpHandleRepoIface *repo,
           handle, handle_str);
     }
 
-  g_string_append (str, TP_ANSI_BOLD_ON "]");
+  g_string_append (str, "]");
 
   return g_string_free (str, FALSE);
 }
@@ -1463,15 +1483,14 @@ emit_members_changed_signals (GObject *channel,
       local_str = member_array_to_string (mixin->handle_repo, local_pending);
       remote_str = member_array_to_string (mixin->handle_repo, remote_pending);
 
-      printf (TP_ANSI_BOLD_ON TP_ANSI_FG_CYAN
-              "%s: emitting members changed\n"
+      printf ("%s: emitting members changed\n"
               "  message       : \"%s\"\n"
               "  added         : %s\n"
               "  removed       : %s\n"
               "  local_pending : %s\n"
               "  remote_pending: %s\n"
               "  actor         : %u\n"
-              "  reason        : %u: %s\n" TP_ANSI_RESET,
+              "  reason        : %u: %s\n",
               G_STRFUNC, message, add_str, rem_str, local_str, remote_str,
               actor, reason, group_change_reason_str (reason));
 
@@ -1804,7 +1823,7 @@ tp_group_mixin_change_members (GObject *obj,
  *  signals were emitted; %FALSE if nothing actually changed and the signals
  *  were suppressed.
  *
- * @since 0.7.21
+ * Since: 0.7.21
  */
 gboolean
 tp_group_mixin_change_members_detailed (GObject *obj,
@@ -1923,7 +1942,7 @@ add_handle_owners_helper (gpointer key,
  * handle to the members, local-pending members or remote-pending members, you
  * must call either this function or tp_group_mixin_add_handle_owner().
  *
- * @since 0.7.10
+ * Since: 0.7.10
  */
 void
 tp_group_mixin_add_handle_owners (GObject *obj,
