@@ -50,8 +50,11 @@
 #include "telepathy-glib/handle-channels-context-internal.h"
 
 #include <telepathy-glib/channel.h>
+#include <telepathy-glib/channel-request.h>
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/gtypes.h>
+#include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/util-internal.h>
 
 #define DEBUG_FLAG TP_DEBUG_CLIENT
 #include "telepathy-glib/debug-internal.h"
@@ -723,3 +726,30 @@ tp_handle_channels_context_get_handler_info (TpHandleChannelsContext *self)
   g_return_val_if_fail (TP_IS_HANDLE_CHANNELS_CONTEXT (self), NULL);
   return self->handler_info;
 }
+
+/**
+ * tp_handle_channels_context_get_requests:
+ * @self: a channel-handling context
+ *
+ * Return a list of the #TpChannelRequest which have been satisfied by the
+ * channels associated with #self.
+ *
+ * Returns: (transfer full) (element-type TelepathyGLib.ChannelRequest):
+ *  a newly allocated #GList of reffed #TpChannelRequest.
+ *
+ * Since: 0.13.14
+ */
+GList *
+tp_handle_channels_context_get_requests (
+    TpHandleChannelsContext *self)
+{
+  GHashTable *request_props;
+
+  request_props = tp_asv_get_boxed (self->handler_info, "request-properties",
+      TP_HASH_TYPE_OBJECT_IMMUTABLE_PROPERTIES_MAP);
+  if (request_props == NULL)
+    return NULL;
+
+  return _tp_create_channel_request_list (
+      tp_proxy_get_dbus_daemon (self->account), request_props);
+}
