@@ -25,9 +25,12 @@
 
 #include <glib.h>
 
+G_BEGIN_DECLS
+
 typedef struct _TpHeap TpHeap;
 
-TpHeap *tp_heap_new (GCompareFunc comparator, GDestroyNotify destructor);
+TpHeap *tp_heap_new (GCompareFunc comparator, GDestroyNotify destructor)
+  G_GNUC_WARN_UNUSED_RESULT;
 void tp_heap_destroy (TpHeap *heap);
 void tp_heap_clear (TpHeap *heap);
 
@@ -38,4 +41,6 @@ gpointer tp_heap_extract_first (TpHeap *heap);
 
 guint tp_heap_size (TpHeap *heap);
 
+G_END_DECLS
+
 #endif
