@@ -44,7 +44,6 @@
 #include "telepathy-glib/proxy-internal.h"
 
 #include "telepathy-glib/_gen/signals-marshal.h"
-#include "telepathy-glib/_gen/tp-cli-protocol.h"
 #include "telepathy-glib/_gen/tp-cli-protocol-body.h"
 
 #include <string.h>
@@ -140,6 +139,7 @@ struct _TpProtocolPrivate
   gchar *vcard_field;
   gchar *english_name;
   gchar *icon_name;
+  GStrv authentication_types;
   TpCapabilities *capabilities;
 };
 
@@ -152,6 +152,7 @@ enum
     PROP_ICON_NAME,
     PROP_CAPABILITIES,
     PROP_PARAM_NAMES,
+    PROP_AUTHENTICATION_TYPES,
     N_PROPS
 };
 
@@ -274,6 +275,10 @@ tp_protocol_get_property (GObject *object,
       g_value_take_boxed (value, tp_protocol_dup_param_names (self));
       break;
 
+    case PROP_AUTHENTICATION_TYPES:
+      g_value_set_boxed (value, tp_protocol_get_authentication_types (self));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -346,6 +351,12 @@ tp_protocol_dispose (GObject *object)
       self->priv->capabilities = NULL;
     }
 
+  if (self->priv->authentication_types)
+    {
+      g_strfreev (self->priv->authentication_types);
+      self->priv->authentication_types = NULL;
+    }
+
   if (dispose != NULL)
     dispose (object);
 }
@@ -424,6 +435,7 @@ tp_protocol_constructed (GObject *object)
   const gchar *s;
   const GPtrArray *rccs;
   gboolean had_immutables = TRUE;
+  const gchar * const *auth_types = NULL;
 
   if (chain_up != NULL)
     chain_up (object);
@@ -476,6 +488,20 @@ tp_protocol_constructed (GObject *object)
   if (rccs != NULL)
     self->priv->capabilities = _tp_capabilities_new (rccs, FALSE);
 
+  auth_types = tp_asv_get_boxed (
+      self->priv->protocol_properties,
+      TP_PROP_PROTOCOL_AUTHENTICATION_TYPES, G_TYPE_STRV);
+
+  if (auth_types != NULL)
+    {
+      self->priv->authentication_types = g_strdupv ((GStrv) auth_types);
+    }
+  else
+    {
+      gchar *tmp[] = { NULL };
+      self->priv->authentication_types = g_strdupv (tmp);
+    }
+
   /* become ready immediately */
   _tp_proxy_set_feature_prepared (proxy, TP_PROTOCOL_FEATURE_PARAMETERS,
       had_immutables);
@@ -603,7 +629,7 @@ tp_protocol_class_init (TpProtocolClass *klass)
    * The name of an icon in the system's icon theme. If none was supplied
    * by the Protocol, or the %TP_PROTOCOL_FEATURE_CORE feature has not been
    * prepared, a default is used; currently, this is "im-" plus
-   * #TpProtocol:name.
+   * #TpProtocol:protocol-name.
    *
    * Since: 0.11.11
    */
@@ -643,6 +669,22 @@ tp_protocol_class_init (TpProtocolClass *klass)
         "A list of parameter names",
         G_TYPE_STRV, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * TpProtocol:authentication-types:
+   *
+   * A non-%NULL #GStrv of interfaces which provide information as to
+   * what kind of authentication channels can possibly appear before
+   * the connection reaches the CONNECTED state, or %NULL if
+   * %TP_PROTOCOL_FEATURE_CORE has not been prepared.
+   *
+   * Since: 0.13.9
+   */
+  g_object_class_install_property (object_class, PROP_AUTHENTICATION_TYPES,
+      g_param_spec_boxed ("authentication-types",
+        "AuthenticationTypes",
+        "A list of authentication types",
+        G_TYPE_STRV, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   proxy_class->list_features = tp_protocol_list_features;
   proxy_class->must_have_unique_name = FALSE;
   proxy_class->interface = TP_IFACE_QUARK_PROTOCOL;
@@ -893,6 +935,24 @@ tp_protocol_get_icon_name (TpProtocol *self)
   return self->priv->icon_name;
 }
 
+/**
+ * tp_protocol_get_authentication_types
+ * @self: a protocol object
+ *
+ *
+ <!-- -->
+ *
+ * Returns: (transfer none): the value of #TpProtocol:authentication-types
+ *
+ * Since: 0.13.9
+ */
+const gchar * const *
+tp_protocol_get_authentication_types (TpProtocol *self)
+{
+  g_return_val_if_fail (TP_IS_PROTOCOL (self), NULL);
+  return (const gchar * const *) self->priv->authentication_types;
+}
+
 /**
  * tp_protocol_get_capabilities:
  * @self: a protocol object
@@ -1391,6 +1451,9 @@ _tp_protocol_parse_manager_file (GKeyFile *file,
   tp_asv_take_string (immutables, TP_PROP_PROTOCOL_ICON,
       replace_null_with_empty (
         g_key_file_get_string (file, group, "Icon", NULL)));
+  tp_asv_take_boxed (immutables, TP_PROP_PROTOCOL_AUTHENTICATION_TYPES,
+      G_TYPE_STRV, g_key_file_get_string_list (file, group,
+          "AuthenticationTypes", NULL, NULL));
 
   rccs = g_ptr_array_new ();
 
