@@ -374,12 +374,14 @@ got_sender_contact_by_handle_cb (TpConnection *connection,
   else if (n_failed > 0)
     {
       DEBUG ("Failed to prepare TpContact (InvalidHandle)");
-
+    }
+  else if (n_contacts > 0)
+    {
       sender = contacts[0];
     }
   else
     {
-      DEBUG ("TpContact of the sender hasn't be prepared");
+      DEBUG ("TpContact of the sender hasn't been prepared");
     }
 
   add_message_received (self, parts, sender, TRUE);
@@ -404,8 +406,14 @@ got_sender_contact_by_id_cb (TpConnection *connection,
     {
       DEBUG ("Failed to prepare TpContact: %s", error->message);
     }
-  else if (n_contacts != 1)
+  else if (n_contacts > 0)
     {
+      sender = contacts[0];
+    }
+  else
+    {
+      DEBUG ("TpContact of the sender hasn't be prepared");
+
       if (DEBUGGING)
         {
           GHashTableIter iter;
@@ -418,12 +426,6 @@ got_sender_contact_by_id_cb (TpConnection *connection,
                   (const gchar *) key, ((GError *) value)->message);
             }
         }
-
-      sender = contacts[0];
-    }
-  else
-    {
-      DEBUG ("TpContact of the sender hasn't be prepared");
     }
 
   add_message_received (self, parts, sender, TRUE);
