@@ -122,6 +122,8 @@ tp_g_ptr_array_contains (GPtrArray *haystack, gpointer needle)
 {
   guint i;
 
+  g_return_val_if_fail (haystack != NULL, FALSE);
+
   for (i = 0; i < haystack->len; i++)
     {
       if (g_ptr_array_index (haystack, i) == needle)
@@ -131,6 +133,38 @@ tp_g_ptr_array_contains (GPtrArray *haystack, gpointer needle)
   return FALSE;
 }
 
+static void
+add_to_array (gpointer data,
+    gpointer user_data)
+{
+  g_ptr_array_add (user_data, data);
+}
+
+/**
+ * tp_g_ptr_array_extend:
+ * @target: a #GPtrArray to copy items to
+ * @source: a #GPtrArray to copy items from
+ *
+ * Appends all elements of @source to @target. Note that this only copies the
+ * pointers from @source; any duplication or reference-incrementing must be
+ * performed by the caller.
+ *
+ * After this function has been called, it is safe to call
+ * g_ptr_array_free() on @source and also free the actual pointer array,
+ * as long as doing so does not free the data pointed to by the new
+ * items in @target.
+ *
+ * Since: 0.14.3
+ */
+void
+tp_g_ptr_array_extend (GPtrArray *target,
+    GPtrArray *source)
+{
+  g_return_if_fail (source != NULL);
+  g_return_if_fail (target != NULL);
+
+  g_ptr_array_foreach (source, add_to_array, target);
+}
 
 /**
  * tp_g_value_slice_new:
