@@ -396,7 +396,7 @@ get_observer_prop_cb (TpProxy *proxy,
 {
   Test *test = user_data;
   GPtrArray *filters;
-  gboolean recover;
+  gboolean recover, delay;
   gboolean valid;
 
   if (error != NULL)
@@ -405,7 +405,7 @@ get_observer_prop_cb (TpProxy *proxy,
       goto out;
     }
 
-  g_assert_cmpint (g_hash_table_size (properties), == , 2);
+  g_assert_cmpint (g_hash_table_size (properties), == , 3);
 
   filters = tp_asv_get_boxed (properties, "ObserverChannelFilter",
       TP_ARRAY_TYPE_CHANNEL_CLASS_LIST);
@@ -415,6 +415,10 @@ get_observer_prop_cb (TpProxy *proxy,
   g_assert (valid);
   g_assert (recover);
 
+  delay = tp_asv_get_boolean (properties, "DelayApprovers", &valid);
+  g_assert (valid);
+  g_assert (delay);
+
 out:
   g_main_loop_quit (test->mainloop);
 }
@@ -490,6 +494,7 @@ test_observer (Test *test,
         NULL));
 
   tp_base_client_set_observer_recover (test->base_client, TRUE);
+  tp_base_client_set_observer_delay_approvers (test->base_client, TRUE);
 
   tp_base_client_register (test->base_client, &test->error);
   g_assert_no_error (test->error);
@@ -1089,6 +1094,7 @@ main (int argc,
       char **argv)
 {
   g_type_init ();
+  tp_tests_abort_after (10);
   tp_debug_set_flags ("all");
 
   g_test_init (&argc, &argv, NULL);
