@@ -28,21 +28,6 @@
 
 #include "tests/lib/util.h"
 
-/* FIXME: if this isn't needed for Senders, remove it */
-G_GNUC_UNUSED static void
-test_assert_uu_hash_contains (GHashTable *hash,
-                              guint key,
-                              guint expected)
-{
-  gpointer v;
-
-  if (!g_hash_table_lookup_extended (hash, GUINT_TO_POINTER (key), NULL, &v))
-    g_error ("Expected %u => %u in hash table, but key was absent", key,
-        expected);
-
-  g_assert_cmpuint (GPOINTER_TO_UINT (v), ==, expected);
-}
-
 typedef struct
 {
   GMainLoop *mainloop;
@@ -66,7 +51,8 @@ typedef struct
   GArray *stream_ids;
   GArray *contacts;
   GPtrArray *get_contents_return;
-  GHashTable *get_senders_return;
+  GHashTable *get_members_return;
+  guint uint_return;
 
   gulong members_changed_detailed_id;
 
@@ -247,7 +233,7 @@ got_contents_cb (TpProxy *proxy,
 }
 
 static void
-got_senders_cb (TpProxy *proxy,
+got_members_cb (TpProxy *proxy,
     const GValue *value,
     const GError *error,
     gpointer user_data,
@@ -255,7 +241,7 @@ got_senders_cb (TpProxy *proxy,
 {
   Test *test = user_data;
 
-  tp_clear_pointer (&test->get_senders_return, g_hash_table_unref);
+  tp_clear_pointer (&test->get_members_return, g_hash_table_unref);
 
   if (test->error != NULL)
     g_clear_error (&test->error);
@@ -263,7 +249,27 @@ got_senders_cb (TpProxy *proxy,
   g_assert_no_error ((GError *) error);
 
   g_assert (G_VALUE_HOLDS (value, FUTURE_HASH_TYPE_CONTACT_SENDING_STATE_MAP));
-  test->get_senders_return = g_value_dup_boxed (value);
+  test->get_members_return = g_value_dup_boxed (value);
+
+  g_main_loop_quit (test->mainloop);
+}
+
+static void
+got_uint_cb (TpProxy *proxy,
+    const GValue *value,
+    const GError *error,
+    gpointer user_data,
+    GObject *weak_object G_GNUC_UNUSED)
+{
+  Test *test = user_data;
+
+  if (test->error != NULL)
+    g_clear_error (&test->error);
+
+  g_assert_no_error ((GError *) error);
+
+  g_assert (G_VALUE_HOLDS (value, G_TYPE_UINT));
+  test->uint_return = g_value_get_uint (value);
 
   g_main_loop_quit (test->mainloop);
 }
@@ -352,8 +358,9 @@ assert_call_properties (GHashTable *get_all_return,
   g_assert_cmpint (tp_asv_get_boolean (get_all_return,
         "MutableContents", &valid), ==, TRUE);
   g_assert (valid);
-  g_assert_cmpstr (tp_asv_get_string (get_all_return,
-        "InitialTransport"), ==, "");
+  g_assert_cmpuint (tp_asv_get_uint32 (get_all_return,
+        "InitialTransport", &valid), ==, FUTURE_STREAM_TRANSPORT_TYPE_UNKNOWN);
+  g_assert (valid);
 
   if (check_call_flags)
     {
@@ -379,7 +386,6 @@ assert_call_properties (GHashTable *get_all_return,
 static void
 assert_content_properties (GHashTable *get_all_return,
     TpMediaStreamType type,
-    TpHandle creator,
     FutureCallContentDisposition disposition)
 {
   gboolean valid;
@@ -388,8 +394,6 @@ assert_content_properties (GHashTable *get_all_return,
   g_assert_cmpuint (tp_asv_get_uint32 (get_all_return, "Type", &valid),
       ==, type);
   g_assert_cmpint (valid, ==, TRUE);
-  g_assert_cmpuint (tp_asv_get_uint32 (get_all_return, "Creator",
-        &valid), ==, creator);
   g_assert_cmpint (valid, ==, TRUE);
   g_assert_cmpuint (tp_asv_get_uint32 (get_all_return, "Disposition",
         &valid), ==, disposition);
@@ -508,7 +512,7 @@ test_basics (Test *test,
   g_main_loop_run (test->mainloop);
   g_assert_no_error (test->error);
   assert_content_properties (test->get_all_return,
-      TP_MEDIA_STREAM_TYPE_AUDIO, test->self_handle,
+      TP_MEDIA_STREAM_TYPE_AUDIO,
       FUTURE_CALL_CONTENT_DISPOSITION_INITIAL);
 
   stream_paths = tp_asv_get_boxed (test->get_all_return, "Streams",
@@ -522,23 +526,29 @@ test_basics (Test *test,
   g_assert (test->audio_stream != NULL);
 
   tp_cli_dbus_properties_call_get (test->audio_stream, -1,
-      FUTURE_IFACE_CALL_STREAM, "Senders", got_senders_cb, test, NULL, NULL);
+      FUTURE_IFACE_CALL_STREAM, "RemoteMembers",
+      got_members_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-
   g_assert_no_error (test->error);
 
-  g_assert_cmpuint (g_hash_table_size (test->get_senders_return), ==, 2);
-  g_assert (!g_hash_table_lookup_extended (test->get_senders_return,
+  g_assert_cmpuint (g_hash_table_size (test->get_members_return), ==, 1);
+  g_assert (!g_hash_table_lookup_extended (test->get_members_return,
         GUINT_TO_POINTER (0), NULL, NULL));
-  g_assert (g_hash_table_lookup_extended (test->get_senders_return,
-        GUINT_TO_POINTER (test->self_handle), NULL, &v));
-  g_assert_cmpuint (GPOINTER_TO_UINT (v), ==, FUTURE_SENDING_STATE_SENDING);
-  g_assert (g_hash_table_lookup_extended (test->get_senders_return,
+  g_assert (!g_hash_table_lookup_extended (test->get_members_return,
+        GUINT_TO_POINTER (test->self_handle), NULL, NULL));
+  g_assert (g_hash_table_lookup_extended (test->get_members_return,
         GUINT_TO_POINTER (tp_channel_get_handle (test->chan, NULL)),
         NULL, &v));
   g_assert_cmpuint (GPOINTER_TO_UINT (v), ==,
       FUTURE_SENDING_STATE_PENDING_SEND);
 
+  tp_cli_dbus_properties_call_get (test->audio_stream, -1,
+      FUTURE_IFACE_CALL_STREAM, "LocalSendingState",
+      got_uint_cb, test, NULL, NULL);
+  g_main_loop_run (test->mainloop);
+  g_assert_no_error (test->error);
+  g_assert_cmpuint (test->uint_return, ==, FUTURE_SENDING_STATE_SENDING);
+
   /* OK, that looks good. Actually make the call */
   future_cli_channel_type_call_call_accept (test->chan, -1, void_cb,
       test, NULL, NULL);
@@ -586,23 +596,30 @@ test_basics (Test *test,
       ==, tp_proxy_get_object_path (test->audio_content));
 
   /* Other contact is sending now */
-  tp_clear_pointer (&test->get_senders_return, g_hash_table_unref);
+  tp_clear_pointer (&test->get_members_return, g_hash_table_unref);
   tp_cli_dbus_properties_call_get (test->audio_stream, -1,
-      FUTURE_IFACE_CALL_STREAM, "Senders", got_senders_cb, test, NULL, NULL);
+      FUTURE_IFACE_CALL_STREAM, "RemoteMembers", got_members_cb, test,
+      NULL, NULL);
   g_main_loop_run (test->mainloop);
   g_assert_no_error (test->error);
 
-  g_assert_cmpuint (g_hash_table_size (test->get_senders_return), ==, 2);
-  g_assert (!g_hash_table_lookup_extended (test->get_senders_return,
+  g_assert_cmpuint (g_hash_table_size (test->get_members_return), ==, 1);
+  g_assert (!g_hash_table_lookup_extended (test->get_members_return,
         GUINT_TO_POINTER (0), NULL, NULL));
-  g_assert (g_hash_table_lookup_extended (test->get_senders_return,
-        GUINT_TO_POINTER (test->self_handle), NULL, &v));
-  g_assert_cmpuint (GPOINTER_TO_UINT (v), ==, FUTURE_SENDING_STATE_SENDING);
-  g_assert (g_hash_table_lookup_extended (test->get_senders_return,
+  g_assert (!g_hash_table_lookup_extended (test->get_members_return,
+        GUINT_TO_POINTER (test->self_handle), NULL, NULL));
+  g_assert (g_hash_table_lookup_extended (test->get_members_return,
         GUINT_TO_POINTER (tp_channel_get_handle (test->chan, NULL)),
         NULL, &v));
   g_assert_cmpuint (GPOINTER_TO_UINT (v), ==, FUTURE_SENDING_STATE_SENDING);
 
+  tp_cli_dbus_properties_call_get (test->audio_stream, -1,
+      FUTURE_IFACE_CALL_STREAM, "LocalSendingState",
+      got_uint_cb, test, NULL, NULL);
+  g_main_loop_run (test->mainloop);
+  g_assert_no_error (test->error);
+  g_assert_cmpuint (test->uint_return, ==, FUTURE_SENDING_STATE_SENDING);
+
   /* AddContent with bad content-type must fail */
 
   future_cli_channel_type_call_call_add_content (test->chan, -1,
@@ -653,7 +670,7 @@ test_basics (Test *test,
   g_main_loop_run (test->mainloop);
   g_assert_no_error (test->error);
   assert_content_properties (test->get_all_return,
-      TP_MEDIA_STREAM_TYPE_VIDEO, test->self_handle,
+      TP_MEDIA_STREAM_TYPE_VIDEO,
       FUTURE_CALL_CONTENT_DISPOSITION_NONE);
 
   stream_paths = tp_asv_get_boxed (test->get_all_return, "Streams",
@@ -666,18 +683,25 @@ test_basics (Test *test,
       g_ptr_array_index (stream_paths, 0), NULL);
   g_assert (test->video_stream != NULL);
 
+  tp_cli_dbus_properties_call_get (test->audio_stream, -1,
+      FUTURE_IFACE_CALL_STREAM, "LocalSendingState",
+      got_uint_cb, test, NULL, NULL);
+  g_main_loop_run (test->mainloop);
+  g_assert_no_error (test->error);
+  g_assert_cmpuint (test->uint_return, ==, FUTURE_SENDING_STATE_SENDING);
+
   tp_cli_dbus_properties_call_get (test->video_stream, -1,
-      FUTURE_IFACE_CALL_STREAM, "Senders", got_senders_cb, test, NULL, NULL);
+      FUTURE_IFACE_CALL_STREAM, "RemoteMembers", got_members_cb, test,
+      NULL, NULL);
   g_main_loop_run (test->mainloop);
   g_assert_no_error (test->error);
 
-  g_assert_cmpuint (g_hash_table_size (test->get_senders_return), ==, 2);
-  g_assert (!g_hash_table_lookup_extended (test->get_senders_return,
+  g_assert_cmpuint (g_hash_table_size (test->get_members_return), ==, 1);
+  g_assert (!g_hash_table_lookup_extended (test->get_members_return,
         GUINT_TO_POINTER (0), NULL, NULL));
-  g_assert (g_hash_table_lookup_extended (test->get_senders_return,
-        GUINT_TO_POINTER (test->self_handle), NULL, &v));
-  g_assert_cmpuint (GPOINTER_TO_UINT (v), ==, FUTURE_SENDING_STATE_SENDING);
-  g_assert (g_hash_table_lookup_extended (test->get_senders_return,
+  g_assert (!g_hash_table_lookup_extended (test->get_members_return,
+        GUINT_TO_POINTER (test->self_handle), NULL, NULL));
+  g_assert (g_hash_table_lookup_extended (test->get_members_return,
         GUINT_TO_POINTER (tp_channel_get_handle (test->chan, NULL)),
         NULL, &v));
 
@@ -1036,7 +1060,7 @@ teardown (Test *test,
   tp_clear_pointer (&test->get_all_return, g_hash_table_unref);
 
   tp_clear_boxed (TP_ARRAY_TYPE_OBJECT_PATH_LIST, &test->get_contents_return);
-  tp_clear_pointer (&test->get_senders_return, g_hash_table_unref);
+  tp_clear_pointer (&test->get_members_return, g_hash_table_unref);
 
   tp_clear_object (&test->audio_stream);
   tp_clear_object (&test->video_stream);
@@ -1061,6 +1085,7 @@ int
 main (int argc,
       char **argv)
 {
+  tp_tests_abort_after (10);
   g_set_prgname ("call-example");
   g_test_init (&argc, &argv, NULL);
   g_test_bug_base ("http://bugs.freedesktop.org/show_bug.cgi?id=");
