@@ -193,6 +193,7 @@ test_prepare (Test *test,
   TpConnectionStatusReason reason;
   TpCapabilities *caps;
   GPtrArray *classes;
+  gchar *cm_name, *protocol_name;
 
   test->conn = tp_connection_new (test->dbus, test->conn_name, test->conn_path,
       &error);
@@ -224,6 +225,20 @@ test_prepare (Test *test,
   g_assert_cmpint (tp_connection_get_status (test->conn, NULL), ==,
       TP_CONNECTION_STATUS_DISCONNECTED);
 
+  g_assert_cmpstr (tp_connection_get_connection_manager_name (test->conn), ==,
+          "simple");
+  g_assert_cmpstr (tp_connection_get_protocol_name (test->conn), ==,
+          "simple-protocol");
+
+  g_object_get (test->conn,
+      "connection-manager-name", &cm_name,
+      "protocol-name", &protocol_name,
+      NULL);
+  g_assert_cmpstr (cm_name, ==, "simple");
+  g_assert_cmpstr (protocol_name, ==, "simple-protocol");
+  g_free (cm_name);
+  g_free (protocol_name);
+
   tp_cli_connection_call_connect (test->conn, -1, NULL, NULL, NULL, NULL);
 
   tp_proxy_prepare_async (test->conn, features, connection_prepared_cb, test);
@@ -384,6 +399,7 @@ int
 main (int argc,
       char **argv)
 {
+  tp_tests_abort_after (10);
   g_test_init (&argc, &argv, NULL);
 
   g_test_add ("/conn/prepare", Test, NULL, setup, test_prepare, teardown);
