@@ -20,12 +20,6 @@
 #include "tests/lib/myassert.h"
 #include "tests/lib/util.h"
 
-typedef struct {
-    TpBaseConnection *base_connection;
-    TpTestsContactsConnection *legacy_service_conn;
-    TpConnection *legacy_client_conn;
-} Fixture;
-
 typedef struct {
     GMainLoop *loop;
     GError *error /* initialized to 0 */;
@@ -35,15 +29,23 @@ typedef struct {
     GHashTable *bad_ids;
 } Result;
 
+typedef struct {
+    Result result;
+    TpBaseConnection *base_connection;
+    TpTestsContactsConnection *legacy_service_conn;
+    TpConnection *legacy_client_conn;
+    TpHandleRepoIface *service_repo;
+} Fixture;
+
 static void
 reset_result (Result *result)
 {
-  /* clean up before doing the second request */
-  g_array_free (result->invalid, TRUE);
-  result->invalid = NULL;
-  g_ptr_array_foreach (result->contacts, (GFunc) g_object_unref, NULL);
-  g_ptr_array_free (result->contacts, TRUE);
-  result->contacts = NULL;
+  tp_clear_pointer (&result->invalid, g_array_unref);
+  if (result->contacts != NULL)
+    g_ptr_array_foreach (result->contacts, (GFunc) g_object_unref, NULL);
+  tp_clear_pointer (&result->contacts, g_ptr_array_unref);
+  tp_clear_pointer (&result->good_ids, g_strfreev);
+  tp_clear_pointer (&result->bad_ids, g_hash_table_unref);
   g_clear_error (&result->error);
 }
 
@@ -236,13 +238,9 @@ test_by_handle (Fixture *f,
 
   /* unref all the handles we created service-side */
   tp_handle_unref (service_repo, handles[0]);
-  MYASSERT (!tp_handle_is_valid (service_repo, handles[0], NULL), "");
   tp_handle_unref (service_repo, handles[1]);
-  MYASSERT (!tp_handle_is_valid (service_repo, handles[1], NULL), "");
   tp_handle_unref (service_repo, handles[2]);
-  MYASSERT (!tp_handle_is_valid (service_repo, handles[2], NULL), "");
   tp_handle_unref (service_repo, handles[3]);
-  MYASSERT (!tp_handle_is_valid (service_repo, handles[3], NULL), "");
 
   /* remaining cleanup */
   g_main_loop_unref (result.loop);
@@ -317,7 +315,6 @@ test_no_features (Fixture *f,
       g_object_unref (contacts[i]);
       tp_tests_proxy_run_until_dbus_queue_processed (client_conn);
       tp_handle_unref (service_repo, handles[i]);
-      MYASSERT (!tp_handle_is_valid (service_repo, handles[i], NULL), "");
     }
 
   /* remaining cleanup */
@@ -484,7 +481,7 @@ test_upgrade (Fixture *f,
 
   tp_connection_upgrade_contacts (client_conn,
       3, contacts,
-      sizeof (features) / sizeof (features[0]), features,
+      G_N_ELEMENTS (features), features,
       upgrade_cb,
       &result, finish, NULL);
 
@@ -541,7 +538,6 @@ test_upgrade (Fixture *f,
       g_object_unref (contacts[i]);
       tp_tests_proxy_run_until_dbus_queue_processed (client_conn);
       tp_handle_unref (service_repo, handles[i]);
-      MYASSERT (!tp_handle_is_valid (service_repo, handles[i], NULL), "");
     }
 
   /* remaining cleanup */
@@ -684,7 +680,7 @@ test_features (Fixture *f,
 
   tp_connection_get_contacts_by_handle (client_conn,
       3, handles,
-      sizeof (features) / sizeof (features[0]), features,
+      G_N_ELEMENTS (features), features,
       by_handle_cb,
       &result, finish, NULL);
 
@@ -832,7 +828,6 @@ test_features (Fixture *f,
       g_object_unref (contacts[i]);
       tp_tests_proxy_run_until_dbus_queue_processed (client_conn);
       tp_handle_unref (service_repo, handles[i]);
-      MYASSERT (!tp_handle_is_valid (service_repo, handles[i], NULL), "");
     }
 
   /* remaining cleanup */
@@ -1207,6 +1202,62 @@ test_by_handle_upgrade (Fixture *f,
   g_main_loop_unref (result.loop);
 }
 
+static void
+test_dup_if_possible (Fixture *f,
+    gconstpointer unused G_GNUC_UNUSED)
+{
+  TpHandle alice_handle, bob_handle;
+  TpContact *alice;
+  TpContact *contact;
+
+  alice_handle = tp_handle_ensure (f->service_repo, "alice", NULL, NULL);
+  g_assert_cmpuint (alice_handle, !=, 0);
+  bob_handle = tp_handle_ensure (f->service_repo, "bob", NULL, NULL);
+  g_assert_cmpuint (bob_handle, !=, 0);
+
+  tp_connection_get_contacts_by_handle (f->legacy_client_conn,
+      1, &alice_handle,
+      0, NULL,
+      by_handle_cb,
+      &f->result, finish, NULL);
+  g_main_loop_run (f->result.loop);
+  g_assert_cmpuint (f->result.contacts->len, ==, 1);
+  g_assert_cmpuint (f->result.invalid->len, ==, 0);
+  g_assert_no_error (f->result.error);
+
+  g_assert (g_ptr_array_index (f->result.contacts, 0) != NULL);
+  alice = g_object_ref (g_ptr_array_index (f->result.contacts, 0));
+  g_assert_cmpuint (tp_contact_get_handle (alice), ==, alice_handle);
+  g_assert_cmpstr (tp_contact_get_identifier (alice), ==, "alice");
+
+  reset_result (&f->result);
+
+  /* we already have a cached TpContact for Alice, so we can get another
+   * copy of it synchronously */
+
+  contact = tp_connection_dup_contact_if_possible (f->legacy_client_conn,
+      alice_handle, "alice");
+  g_assert (contact == alice);
+  g_object_unref (contact);
+
+  contact = tp_connection_dup_contact_if_possible (f->legacy_client_conn,
+      alice_handle, NULL);
+  g_assert (contact == alice);
+  g_object_unref (contact);
+
+  /* because this connection pretends not to have immortal handles, we can't
+   * reliably get a contact for Bob synchronously, even if we supply his
+   * identifier */
+
+  contact = tp_connection_dup_contact_if_possible (f->legacy_client_conn,
+      bob_handle, "bob");
+  g_assert (contact == NULL);
+
+  contact = tp_connection_dup_contact_if_possible (f->legacy_client_conn,
+      bob_handle, NULL);
+  g_assert (contact == NULL);
+}
+
 static void
 setup (Fixture *f,
     gconstpointer unused G_GNUC_UNUSED)
@@ -1216,6 +1267,9 @@ setup (Fixture *f,
 
   f->legacy_service_conn = g_object_ref (TP_TESTS_CONTACTS_CONNECTION (
         f->base_connection));
+  f->service_repo = tp_base_connection_get_handles (f->base_connection,
+      TP_HANDLE_TYPE_CONTACT);
+  f->result.loop = g_main_loop_new (NULL, FALSE);
 }
 
 static void
@@ -1233,9 +1287,12 @@ teardown (Fixture *f,
       g_assert (ok);
     }
 
+  f->service_repo = NULL;
   tp_clear_object (&f->legacy_client_conn);
   tp_clear_object (&f->legacy_service_conn);
   tp_clear_object (&f->base_connection);
+  reset_result (&f->result);
+  tp_clear_pointer (&f->result.loop, g_main_loop_unref);
 }
 
 int
@@ -1244,6 +1301,7 @@ main (int argc,
 {
   g_type_init ();
   tp_debug_set_flags ("all");
+  tp_tests_abort_after (10);
   g_set_prgname ("contacts-slow-path");
   g_test_init (&argc, &argv, NULL);
   g_test_bug_base ("http://bugs.freedesktop.org/show_bug.cgi?id=");
@@ -1262,6 +1320,8 @@ main (int argc,
       test_by_handle_again, teardown);
   g_test_add ("/contacts-slow-path/by-handle-upgrade", Fixture, NULL, setup,
       test_by_handle_upgrade, teardown);
+  g_test_add ("/contacts-slow-path/dup-if-possible", Fixture, NULL, setup,
+      test_dup_if_possible, teardown);
 
   return g_test_run ();
 }
