@@ -265,7 +265,6 @@ test_contact_info (Fixture *f,
   TpHandleRepoIface *service_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) service_conn, TP_HANDLE_TYPE_CONTACT);
   TpContactFeature features[] = { TP_CONTACT_FEATURE_CONTACT_INFO };
-  TpContact *self_contact;
   TpContact *contact;
   TpHandle handle;
   const gchar *field_value[] = { "Foo", NULL };
@@ -308,7 +307,7 @@ test_contact_info (Fixture *f,
       &result, finish, NULL);
   g_main_loop_run (result.loop);
   g_assert_no_error (result.error);
-  self_contact = g_object_ref (g_ptr_array_index (result.contacts, 0));
+  g_object_ref (g_ptr_array_index (result.contacts, 0));
   reset_result (&result);
 
   tp_connection_set_contact_info_async (client_conn, info_list,
