@@ -4,6 +4,8 @@
 #include <telepathy-glib/debug.h>
 #include <telepathy-glib/util.h>
 
+#include "tests/lib/util.h"
+
 static void
 test_validation (void)
 {
@@ -330,11 +332,12 @@ int
 main (int argc,
       char **argv)
 {
+  tp_tests_abort_after (10);
+  tp_debug_set_flags ("all");
+
   g_type_init ();
   g_test_init (&argc, &argv, NULL);
 
-  tp_debug_set_flags ("all");
-
   g_test_add_func ("/dbus/validation", test_validation);
   g_test_add_func ("/dbus-daemon/properties", test_properties);
   g_test_add_func ("/dbus-daemon/watch-name-owner", test_watch_name_owner);
