@@ -81,6 +81,8 @@ static guint message_sent_count = 0;
 static guint last_message_sent_type = 0;
 static gchar *last_message_sent_token = NULL;
 static guint last_message_sent_n_parts = 0;
+static guint last_message_sent_sender = 0;
+static gchar *last_message_sent_sender_id = NULL;
 
 static void
 print_part (gpointer k,
@@ -152,10 +154,14 @@ on_message_sent (TpChannel *chan,
   guint i;
   GHashTable *headers = g_ptr_array_index (parts, 0);
   guint type;
+  guint sender;
+  const gchar *sender_id;
 
   g_assert (parts->len >= 1);
 
   type = tp_asv_get_uint32 (headers, "message-type", NULL);
+  sender = tp_asv_get_uint32 (headers, "message-sender", NULL);
+  sender_id = tp_asv_get_string (headers, "message-sender-id");
 
   g_print ("%p: MessageSent with token '%s': type %u, %u parts\n",
       chan, token, type, parts->len);
@@ -171,6 +177,9 @@ on_message_sent (TpChannel *chan,
   last_message_sent_n_parts = parts->len;
   g_free (last_message_sent_token);
   last_message_sent_token = g_strdup (token);
+  g_free (last_message_sent_sender_id);
+  last_message_sent_sender_id = g_strdup (sender_id);
+  last_message_sent_sender = sender;
 }
 
 static void
@@ -207,6 +216,7 @@ main (int argc,
   gboolean ok;
   GHashTable *parameters;
 
+  tp_tests_abort_after (10);
   g_type_init ();
   tp_debug_set_flags ("all");
   dbus = tp_tests_dbus_daemon_dup_or_die ();
@@ -305,6 +315,7 @@ main (int argc,
     {
       const GValue *value;
       gchar *contents;
+      GArray *types;
       GPtrArray *messages;
       GHashTable *properties = NULL;
 
@@ -314,8 +325,7 @@ main (int argc,
 
       g_print ("\n\n==== Examining properties ====\n\n");
 
-      MYASSERT (g_hash_table_size (properties) == 3, "%u",
-          g_hash_table_size (properties));
+      g_assert_cmpuint (g_hash_table_size (properties), ==, 5);
 
       MYASSERT (tp_asv_get_uint32 (properties, "MessagePartSupportFlags", NULL)
           == ( TP_MESSAGE_PART_SUPPORT_FLAG_ONE_ATTACHMENT
@@ -329,12 +339,28 @@ main (int argc,
       g_message ("%s", contents);
       g_free (contents);
 
+      g_assert ((value = tp_asv_lookup (properties, "MessageTypes"))
+          != NULL);
+      g_assert (G_VALUE_HOLDS (value, DBUS_TYPE_G_UINT_ARRAY));
+      types = g_value_get_boxed (value);
+      g_assert_cmpuint (types->len, ==, 3);
+      g_assert_cmpuint (g_array_index (types, guint, 0), ==,
+          TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL);
+      g_assert_cmpuint (g_array_index (types, guint, 1), ==,
+          TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION);
+      g_assert_cmpuint (g_array_index (types, guint, 2), ==,
+          TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE);
+
       MYASSERT ((value = tp_asv_lookup (properties, "PendingMessages"))
           != NULL, "");
       MYASSERT (G_VALUE_HOLDS_BOXED (value), "");
       messages = g_value_get_boxed (value);
       MYASSERT (messages->len == 0, "%u", messages->len);
 
+      g_assert_cmpuint (tp_asv_get_uint32 (properties,
+            "DeliveryReportingSupport", NULL), ==,
+          TP_DELIVERY_REPORTING_SUPPORT_FLAG_RECEIVE_FAILURES);
+
       g_hash_table_destroy (properties);
     }
 
@@ -380,6 +406,9 @@ main (int argc,
   MYASSERT (message_received_count == 1, ": %u != 1", message_received_count);
   MYASSERT (last_message_sent_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
       ": %u != NORMAL", last_message_sent_type);
+  g_assert_cmpuint (last_message_sent_sender, ==,
+      tp_connection_get_self_handle (conn));
+  g_assert_cmpstr (last_message_sent_sender_id, ==, "me@example.com");
   MYASSERT (last_message_sent_n_parts == 2,
       ": %u != 2", last_message_sent_n_parts);
   MYASSERT (last_message_received_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
@@ -421,6 +450,9 @@ main (int argc,
   MYASSERT (message_received_count == 1, ": %u != 1", message_received_count);
   MYASSERT (last_message_sent_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION,
       ": %u != ACTION", last_message_sent_type);
+  g_assert_cmpuint (last_message_sent_sender, ==,
+      tp_connection_get_self_handle (conn));
+  g_assert_cmpstr (last_message_sent_sender_id, ==, "me@example.com");
   MYASSERT (last_message_sent_n_parts == 2,
       ": %u != 2", last_message_sent_n_parts);
   MYASSERT (last_message_received_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION,
@@ -462,6 +494,9 @@ main (int argc,
   MYASSERT (message_received_count == 1, ": %u != 1", message_received_count);
   MYASSERT (last_message_sent_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE,
       ": %u != NOTICE", last_message_sent_type);
+  g_assert_cmpuint (last_message_sent_sender, ==,
+      tp_connection_get_self_handle (conn));
+  g_assert_cmpstr (last_message_sent_sender_id, ==, "me@example.com");
   MYASSERT (last_message_sent_n_parts == 2,
       ": %u != 2", last_message_sent_n_parts);
   MYASSERT (last_message_received_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE,
@@ -562,6 +597,9 @@ main (int argc,
       ": '%s'", last_received_text);
   MYASSERT (message_sent_count == 1, ": %u != 1", message_sent_count);
   MYASSERT (message_received_count == 1, ": %u != 1", message_received_count);
+  g_assert_cmpuint (last_message_sent_sender, ==,
+      tp_connection_get_self_handle (conn));
+  g_assert_cmpstr (last_message_sent_sender_id, ==, "me@example.com");
   MYASSERT (last_message_sent_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
       ": %u != NORMAL", last_message_sent_type);
   MYASSERT (last_message_sent_n_parts == 4,
@@ -670,6 +708,9 @@ main (int argc,
   MYASSERT (message_received_count == 1, ": %u != 1", message_received_count);
   MYASSERT (last_message_sent_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
       ": %u != NORMAL", last_message_sent_type);
+  g_assert_cmpuint (last_message_sent_sender, ==,
+      tp_connection_get_self_handle (conn));
+  g_assert_cmpstr (last_message_sent_sender_id, ==, "me@example.com");
   MYASSERT (last_message_sent_n_parts == 5,
       ": %u != 5", last_message_sent_n_parts);
   MYASSERT (last_message_received_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
@@ -758,6 +799,9 @@ main (int argc,
   MYASSERT (message_received_count == 1, ": %u != 1", message_received_count);
   MYASSERT (last_message_sent_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
       ": %u != NORMAL", last_message_sent_type);
+  g_assert_cmpuint (last_message_sent_sender, ==,
+      tp_connection_get_self_handle (conn));
+  g_assert_cmpstr (last_message_sent_sender_id, ==, "me@example.com");
   MYASSERT (last_message_sent_n_parts == 4,
       ": %u != 4", last_message_sent_n_parts);
   MYASSERT (last_message_received_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
@@ -954,8 +998,7 @@ main (int argc,
 
       g_print ("\n\n==== Examining properties ====\n\n");
 
-      MYASSERT (g_hash_table_size (properties) == 3, "%u",
-          g_hash_table_size (properties));
+      g_assert_cmpuint (g_hash_table_size (properties), ==, 5);
 
       MYASSERT (tp_asv_get_uint32 (properties, "MessagePartSupportFlags", NULL)
           == ( TP_MESSAGE_PART_SUPPORT_FLAG_ONE_ATTACHMENT
@@ -969,6 +1012,10 @@ main (int argc,
       g_message ("%s", contents);
       g_free (contents);
 
+      g_assert_cmpuint (tp_asv_get_uint32 (properties,
+            "DeliveryReportingSupport", NULL), ==,
+          TP_DELIVERY_REPORTING_SUPPORT_FLAG_RECEIVE_FAILURES);
+
       MYASSERT ((value = tp_asv_lookup (properties, "PendingMessages"))
           != NULL, "");
       MYASSERT (G_VALUE_HOLDS_BOXED (value), "");
@@ -1054,6 +1101,7 @@ main (int argc,
   g_free (last_sent_text);
   g_free (last_received_text);
   g_free (last_message_sent_token);
+  g_free (last_message_sent_sender_id);
 
   return 0;
 }
