@@ -220,6 +220,7 @@ get_observer_prop_cb (TpProxy *proxy,
   GPtrArray *filters;
   gboolean recover;
   gboolean valid;
+  gboolean delay;
 
   if (error != NULL)
     {
@@ -227,7 +228,7 @@ get_observer_prop_cb (TpProxy *proxy,
       goto out;
     }
 
-  g_assert_cmpint (g_hash_table_size (properties), == , 2);
+  g_assert_cmpint (g_hash_table_size (properties), == , 3);
 
   filters = tp_asv_get_boxed (properties, "ObserverChannelFilter",
       TP_ARRAY_TYPE_CHANNEL_CLASS_LIST);
@@ -237,6 +238,10 @@ get_observer_prop_cb (TpProxy *proxy,
   g_assert (valid);
   g_assert (recover);
 
+  delay = tp_asv_get_boolean (properties, "DelayApprovers", &valid);
+  g_assert (valid);
+  g_assert (!delay);
+
 out:
   g_main_loop_quit (test->mainloop);
 }
@@ -464,6 +469,7 @@ int
 main (int argc,
       char **argv)
 {
+  tp_tests_abort_after (10);
   g_type_init ();
   tp_debug_set_flags ("all");
 
